@@ -7,6 +7,7 @@ import argparse
 import re
 import sys
 import time
+import yaml
 from io import StringIO
 
 import nbformat
@@ -20,6 +21,7 @@ from sos.utils import env
 # Converter from Notebook
 #
 
+
 def get_notebook_to_script_parser():
     parser = argparse.ArgumentParser(
         'sos convert FILE.ipynb FILE.sos (or --to sos)',
@@ -116,33 +118,6 @@ def get_script_to_notebook_parser():
     return parser
 
 
-def add_cell(cells, content, cell_type, cell_count, metainfo):
-    # if a section consist of all report, report it as a markdown cell
-    if not content:
-        return
-    if cell_type not in ('code', 'markdown'):
-        env.logger.warning(f'Unrecognized cell type {cell_type}, code assumed.')
-    if cell_type == 'markdown' and any(
-            x.strip() and not x.startswith('#! ') for x in content):
-        env.logger.warning(
-            'Markdown lines not starting with #!, code cell assumed.')
-        cell_type = 'code'
-    #
-    if cell_type == 'markdown':
-        cells.append(
-            new_markdown_cell(
-                source=''.join([x[3:] for x in content]).strip(),
-                metadata=metainfo))
-    else:
-        cells.append(
-            new_code_cell(
-                # remove any trailing blank lines...
-                source=''.join(content).strip(),
-                execution_count=cell_count,
-                metadata=metainfo))
-
-
-
 def script_to_notebook(script_file, notebook_file, args=None,
                        unknown_args=None):
     '''
@@ -157,6 +132,32 @@ def script_to_notebook(script_file, notebook_file, args=None,
     metainfo = {}
     content = []
 
+    def add_cell(cells, content, cell_type, cell_count, metainfo):
+        # if a section consist of all report, report it as a markdown cell
+        if not content:
+            return
+        if cell_type not in ('code', 'markdown'):
+            env.logger.warning(
+                f'Unrecognized cell type {cell_type}, code assumed.')
+        if cell_type == 'markdown' and any(
+                x.strip() and not x.startswith('#! ') for x in content):
+            env.logger.warning(
+                'Markdown lines not starting with #!, code cell assumed.')
+            cell_type = 'code'
+        #
+        if cell_type == 'markdown':
+            cells.append(
+                new_markdown_cell(
+                    source=''.join([x[3:] for x in content]).strip(),
+                    metadata=metainfo))
+        else:
+            cells.append(
+                new_code_cell(
+                    # remove any trailing blank lines...
+                    source=''.join(content).strip(),
+                    execution_count=cell_count,
+                    metadata=metainfo))
+
     with open(script_file) as script:
         first_block = True
         for line in script:
@@ -344,11 +345,7 @@ def get_notebook_to_html_parser():
         prominent tag, and a control panel to control the display of the rest of the content
         ''')
     parser.add_argument(
-        '-e',
-        '--execute',
-        action='store_true',
-        help='''Deprecated'''
-    )
+        '-e', '--execute', action='store_true', help='''Deprecated''')
     parser.add_argument(
         '-v',
         '--view',
@@ -365,7 +362,8 @@ def notebook_to_html(notebook_file, output_file, sargs=None, unknown_args=None):
     if unknown_args is None:
         unknown_args = []
     if sargs and sargs.execute:
-        env.logger.warning('Option --execute is deprecated. Please use sos-papermill instead.')
+        env.logger.warning(
+            'Option --execute is deprecated. Please use sos-papermill instead.')
     if sargs.template:
         unknown_args = [
             '--template',
@@ -605,70 +603,167 @@ def Rmarkdown_to_notebook(rmarkdown_file,
                           output_file,
                           sargs=None,
                           unknown_args=None):
+    cells = []
+    code_count = 1
+
     #
     with open(rmarkdown_file) as script:
-        content = script.read()
-    #
-    # identify the headers
-    header = header = re.compile('^(#+\s.*$)', re.M)
-    paragraphs = re.split(header, content)
-    #
-    cells = []
-    cell_count = 1
-    for idx, p in enumerate(paragraphs):
-        if idx % 2 == 1:
-            # this is header, let us create a markdown cell
-            cells.append(new_markdown_cell(source=p.strip()))
-        else:
-            # this is unknown yet, let us find ```{} block
-            code = re.compile('^\s*(```{.*})$', re.M)
-            endcode = re.compile('^\s*```$', re.M)
+        rmdlines = script.readlines()
 
-            for pidx, pc in enumerate(re.split(code, p)):
-                if pidx == 0:
-                    # piece before first code block. it might contain
-                    # inline expression
-                    cells.append(new_markdown_cell(source=pc.strip()))
-                elif pidx % 2 == 0:
-                    # this is AFTER the {r} piece, let us assume all R code
-                    # for now
-                    # this is code, but it should end somewhere
-                    pieces = re.split(endcode, pc)
-                    # I belive that we should have
-                    #   pieces[0] <- code
-                    #   pieces[1] <- rest...
-                    # but I could be wrong.
-                    cells.append(
-                        new_code_cell(
-                            source=pieces[0],
-                            execution_count=cell_count,
-                            metadata={'kernel': 'R'}))
-                    cell_count += 1
-                    #
-                    for piece in pieces[1:]:
-                        cells.append(new_markdown_cell(source=piece.strip()))
+    def add_cell(cells, content, cell_type, metainfo):
+        nonlocal code_count
+        # if a section consist of all report, report it as a markdown cell
+        if not content:
+            return
+        if cell_type not in ('code', 'markdown'):
+            env.logger.warning(
+                f'Unrecognized cell type {cell_type}, code assumed.')
+        #
+        if cell_type == 'code':
+            cells.append(
+                new_code_cell(
+                    # remove any trailing blank lines...
+                    source=''.join(content).strip(),
+                    execution_count=code_count,
+                    metadata=metainfo))
+            code_count += 1
+        elif metainfo.get('kernel', '') == 'Markdown':
+            # markdown code with inline expression
+            cells.append(
+                new_code_cell(
+                    # remove any trailing blank lines...
+                    source=f'%expand `r ` --in R\n' + ''.join(content).strip(),
+                    execution_count=code_count,
+                    metadata=metainfo))
+            code_count += 1
+        else:
+            cells.append(
+                new_markdown_cell(
+                    source=''.join(content).strip(), metadata=metainfo))
+
+    Rmd_header = {}
+    # YAML front matter appears to be restricted to strictly ---\nYAML\n---
+    re_yaml_delim = re.compile(r"^---\s*$")
+    delim_lines = [i for i, l in enumerate(rmdlines) if re_yaml_delim.match(l)]
+    if len(delim_lines) >= 2 and delim_lines[1] - delim_lines[0] > 1:
+        yamltext = '\n'.join(rmdlines[delim_lines[0] + 1:delim_lines[1]])
+        try:
+            Rmd_header = yaml.safe_load(yamltext)
+        except yaml.YAMLError as e:
+            env.logger.warning(f"Error reading document metadata block: {e}")
+            env.logger.warning("Trying to continue without header")
+        rmdlines = rmdlines[:delim_lines[0]] + rmdlines[delim_lines[1] + 1:]
+
+    # the behaviour of rmarkdown appears to be that a code block does not
+    # have to have matching numbers of start and end `s - just >=3
+    # and there can be any number of spaces before the {r, meta} block,
+    # but "r" must be the first character of that block
+
+    re_code_start = re.compile(r"^````*\s*{r(.*)}\s*$")
+    re_code_end = re.compile(r"^````*\s*$")
+    re_code_inline = re.compile(r"`r.+`")
+
+    MD, CODE = range(2)
+
+    state = MD
+    celldata = []
+    meta = {}
+    has_inline_markdown = False
+
+    for l in rmdlines:
+        if state == MD:
+            match = re_code_start.match(l)
+            if match:
+                state = CODE
+                # only add MD cells with non-whitespace content
+                if any([c.strip() for c in celldata]):
+                    add_cell(cells, celldata, 'markdown', metainfo=meta)
+
+                celldata = []
+                meta = {'kernel': 'R'}
+
+                if match.group(1):
+                    chunk_opts = match.group(1).strip(" ,")
+                    if chunk_opts:
+                        meta['Rmd_chunk_options'] = chunk_opts
+                        if 'include=FALSE' in chunk_opts or 'echo=FALSE' in chunk_opts:
+                            if 'jupyter' in meta:
+                                meta['jupyter']['source_hidden'] = True
+                            else:
+                                meta["jupyter"] = {"source_hidden": True}
+                        if 'include=FALSE' in chunk_opts:
+                            if 'jupyter' in meta:
+                                meta['jupyter']['output_hidden'] = True
+                            else:
+                                meta["jupyter"] = {"output_hidden": True}
+            else:
+                if re_code_inline.search(l):
+                    if not meta.get('kernel', '') and any(
+                            c.strip() for c in celldata):
+                        # if there is markdown text before it, see if there are entire paragraphs
+                        # and put in regular markdown cell
+                        last_empty_line = len(celldata) - 1
+                        while last_empty_line > 0:
+                            if celldata[last_empty_line].strip():
+                                last_empty_line -= 1
+                            else:
+                                break
+                        if last_empty_line > 0:
+                            add_cell(
+                                cells,
+                                celldata[:last_empty_line + 1],
+                                'markdown',
+                                metainfo=meta)
+                            celldata = celldata[last_empty_line + 1:]
+                    # inline markdown ...
+                    has_inline_markdown = True
+                    # we use hidden to indicate that the input of this code
+                    # is supposed to be hidden
+                    meta['kernel'] = 'Markdown'
+                    meta["jupyter"] = {"source_hidden": True}
+                # cell.source in ipynb does not include implicit newlines
+                celldata.append(l.rstrip() + "\n")
+        else:  # CODE
+            if re_code_end.match(l):
+                state = MD
+                # unconditionally add code blocks regardless of content
+                add_cell(cells, celldata, 'code', metainfo=meta)
+                celldata = []
+                meta = {}
+            else:
+                if len(celldata) > 0:
+                    celldata[-1] = celldata[-1] + "\n"
+                celldata.append(l.rstrip())
+
+    if state == CODE and any([c.strip() for c in celldata]):
+        add_cell(cells, celldata, 'code', metainfo=meta)
+    elif any([c.strip() for c in celldata]):
+        add_cell(cells, celldata, 'markdown', metainfo=meta)
     #
     # create header
-    nb = new_notebook(
-        cells=cells,
-        metadata={
-            'kernelspec': {
-                "display_name": "SoS",
-                "language": "sos",
-                "name": "sos"
-            },
-            "language_info": {
-                "file_extension": ".sos",
-                "mimetype": "text/x-sos",
-                "name": "sos",
-                "pygments_lexer": "python",
-                'nbconvert_exporter': 'sos_notebook.converter.SoS_Exporter',
-            },
-            'sos': {
-                'kernels': [['SoS', 'sos', '', ''], ['R', 'ir', '', '']],
-                'default_kernel': 'R'
-            }
-        })
+    metadata = {
+        'kernelspec': {
+            "display_name": "SoS",
+            "language": "sos",
+            "name": "sos"
+        },
+        "language_info": {
+            "file_extension": ".sos",
+            "mimetype": "text/x-sos",
+            "name": "sos",
+            "pygments_lexer": "python",
+            'nbconvert_exporter': 'sos_notebook.converter.SoS_Exporter',
+        },
+        'sos': {
+            'kernels': [['SoS', 'sos', '', ''], ['R', 'ir', '', '']]
+        }
+    }
+    if has_inline_markdown:
+        metadata['sos']['kernels'].append(['Markdown', 'markdown', '', ''])
+    if Rmd_header:
+        metadata['Rmd_chunk_options'] = Rmd_header
+
+    nb = new_notebook(cells=cells, metadata=metadata)
 
     if not output_file:
         nbformat.write(nb, sys.stdout, 4)
